<?php

namespace Drupal\media_twig_tools\Services;

use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\file\FileInterface;
use Drupal\image\ImageStyleInterface;
use Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface;
use Drupal\media_twig_tools\Generators\ImageGenerator;
use GuzzleHttp\Client;

/**
 * This class builds image from settings.
 *
 * @package Drupal\media_twig_tools\Services
 */
class ImageBuilder implements ImageBuilderInterface {

  /**
   * Conf.
   *
   * @var \Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface
   */
  protected MediaTwigToolsConfigInterface $mediaTwigToolsConfig;

  /**
   * Bounce.
   *
   * @var \Drupal\media_twig_tools\Services\BounceInterface
   */
  protected BounceInterface $bounce;

  /**
   * File URL Generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Http client.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  /**
   * Image Builder constructor.
   *
   * @param \Drupal\media_twig_tools\Config\MediaTwigToolsConfigInterface $mediaTwigToolsConfig
   *   The config.
   * @param \Drupal\media_twig_tools\Services\BounceInterface $bounce
   *   The bounce.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file url generator.
   * @param \GuzzleHttp\Client $http_client
   *   The http client.
   */
  public function __construct(
    MediaTwigToolsConfigInterface $mediaTwigToolsConfig,
    BounceInterface $bounce,
    FileUrlGeneratorInterface $file_url_generator,
    Client $http_client,
  ) {
    $this->mediaTwigToolsConfig = $mediaTwigToolsConfig;
    $this->bounce = $bounce;
    $this->bounce->setDirname($this->getMediaDirname());
    $this->fileUrlGenerator = $file_url_generator;
    $this->httpClient = $http_client;
  }

  /**
   * Singleton quick access.
   *
   * @return static
   *   Singleton.
   *
   * @deprecated in media_twig_tools:2.1.0 and is removed from media_twig_tools:2.2.0
   *   Use \Drupal:service(MediaTwigToolsInterface::SERVICE_NAME);
   *
   * @see https://www.drupal.org/project/media_twig_tools/issues/3369533
   */
  public static function me() {
    return \Drupal::service(ImageBuilderInterface::SERVICE_NAME);
  }

  /**
   * Return the url image from settings.
   *
   * @param string $fileId
   *   The file id.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromSettings($fileId, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce') {
    $imgUrl = NULL;

    // Get file.
    if ($file = $this->mediaTwigToolsConfig->getFileStorage()
      ->load(intval($fileId))) {

      $style =
        ($srcSetSettings[MediaTwigToolsInterface::OPTION_STYLE] ?? NULL)
        ?? ($srcSettings[MediaTwigToolsInterface::OPTION_STYLE] ?? NULL);

      $url = $this->getStyledUrlFromFileId(
        $file->id(),
        $style,
        isset($options[MediaTwigToolsInterface::OPTION_RELATIVE]) && $options[MediaTwigToolsInterface::OPTION_RELATIVE]
      );

      $imgUrl = $this->getImageUrlFromFileAndSettings($file->id(), $url, $srcSettings, $srcSetSettings, $options, $rule);
    }

    $this->bounce->addBounce($imgUrl);

    return $imgUrl;
  }

  /**
   * Quick replace.
   *
   * @param string $source
   *   The source string.
   * @param array $replace
   *   The replacement [$needle => $replacement].
   *
   * @return array|string|string[]
   *   The string replaced.
   */
  public function replace($source, array $replace) {
    return str_replace(
      array_keys($replace),
      array_values($replace),
      $source,
    );
  }

  /**
   * Return the rules from the dirname.
   *
   * @param string $path
   *   THe path.
   *
   * @return array
   *   The rules.
   */
  public function getRulesFromPathString($path) {
    $rules = [];
    foreach (explode(static::RULES_SEPARATOR, $path) as $data) {
      // Split letters and numbers.
      $split = preg_split("/(,?\s+)|((?<=[a-z])(?=\d))|((?<=\d)(?=[a-z]))/i", $data);

      // Associative array.
      if (is_numeric(reset($split))) {
        $key = 1;
        $delta = -1;
      }
      else {
        $key = 0;
        $delta = 1;
      }
      $rule = [
        'id' => array_pop($split),
        'data' => [],
      ];
      for ($i = $key; $i <= count($split) - $delta; $i += 2) {
        if (isset($split[$i + $delta]) && isset($split[$i])) {
          $rule['data'][$split[$i]] = $split[$i + $delta];
        }
      }

      $rules[] = $rule;
    }

    return $rules;
  }

  /**
   * Return rules data.
   *
   * @param array $srcSettings
   *   The src settings.
   *
   * @return array
   *   The rules data.
   */
  public function getRulesData(array $srcSettings) {

    if (isset($srcSettings[MediaTwigToolsInterface::OPTION_STYLE])
      && $style = $this->mediaTwigToolsConfig->getImageStyleStorage()
        ->load($srcSettings[MediaTwigToolsInterface::OPTION_STYLE])) {
      $srcSettings[MediaTwigToolsInterface::OPTION_STYLE] = $style;
    }
    else {
      unset($srcSettings[MediaTwigToolsInterface::OPTION_STYLE]);
    }

    return $srcSettings;
  }

  /**
   * Return the dirname according to rules data.
   *
   * @param array $rulesData
   *   The rules data.
   *
   * @return string
   *   The rules dirname.
   */
  public function getRulesDirnameFromRulesData(array $rulesData) {
    $rules = [];
    foreach ($rulesData[MediaTwigToolsInterface::OPTION_RULES] as $weight => $rule) {
      $rules[$weight] = '';
      foreach ($rule['data'] as $key => $value) {
        $rules[$weight] .= $key . $value;
      }
      $rules[$weight] .= $rule['id'];
    }

    return implode(static::RULES_SEPARATOR, $rules);
  }

  /**
   * Return true if $path is an image builder route.
   *
   * @param string $path
   *   THe path.
   *
   * @return bool
   *   The status of the image.
   */
  public function isImageBuilderRoute(string $path) {
    $pathInfo = pathinfo($path);
    $pathData = explode('/', $pathInfo['dirname']);
    $mediaDirnameIndex = count($pathData) - 3;

    return isset($pathData[$mediaDirnameIndex]) && $pathData[$mediaDirnameIndex] === $this->getMediaDirname();
  }

  /**
   * Return the build image dirname.
   *
   * @return string
   *   THe media dirname.
   */
  public function getMediaDirname() {
    return $this->mediaTwigToolsConfig->getDirname();
  }

  /**
   * Return the no style name.
   *
   * @return string
   *   THe no style name.
   */
  public function getNoStyleName() {
    return $this->mediaTwigToolsConfig->getNoStyle();
  }

  /**
   * Generat the image and return the generator.
   *
   * @param string $path
   *   The image path.
   *
   * @return \Drupal\media_twig_tools\Generators\ImageGenerator
   *   The Image Generator.
   *
   * @throws \Exception
   */
  public function generateImage(string $path) {
    if ($allowMaxSize = $this->mediaTwigToolsConfig->getAllowMaxSize()) {
      ini_set('memory_limit', $allowMaxSize . 'M');
    }

    // Check generation rights.
    if ($this->mediaTwigToolsConfig->isFormatBounceEnable()) {
      $this->bounce->checkBounce($path);
    }

    $generator = new ImageGenerator($this->getDataFromPath($path), $this);
    $generator->generate();

    return $generator;
  }

  /**
   * Return image builder data from image path.
   *
   * @param string $path
   *   The path.
   *
   * @return array
   *   The data.
   *
   * @throws \Exception
   */
  public function getDataFromPath($path) {
    if (!$this->isImageBuilderRoute($path)) {
      throw new \Exception('Not a media builder path');
    }

    $pathInfo = pathinfo($path);

    // File id.
    $nameData = explode('_', $pathInfo['basename']);
    $pathInfo['f'] = reset($nameData);
    $pathInfo['original_basename'] = implode('_', array_slice($nameData, 1));

    // Settings and options.
    $settings = explode('/', $pathInfo['dirname']);
    $count = count($settings);
    $pathInfo[MediaTwigToolsInterface::OPTION_RULES] = $this->getRulesFromPathString(end($settings));
    $pathInfo[MediaTwigToolsInterface::OPTION_STYLE] = $settings[$count - 2];

    return $pathInfo;
  }

  /**
   * Return the url image from file and settings.
   *
   * @param string $fileId
   *   The file id.
   * @param string $url
   *   The file url.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromFileAndSettings($fileId, $url, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce') {

    // Path data.
    $urlPath = pathinfo($url);
    $dirPath = pathinfo($urlPath['dirname']);

    // Rules.
    $rules = $this->getRulesData($srcSettings);

    // Get style.
    /** @var \Drupal\image\ImageStyleInterface $style */
    $style = $rules[MediaTwigToolsInterface::OPTION_STYLE] ?? NULL;
    if (!$style) {
      if (isset($srcSetSettings[MediaTwigToolsInterface::OPTION_STYLE])) {
        if ($srcSetSettings[MediaTwigToolsInterface::OPTION_STYLE] instanceof ImageStyleInterface) {
          $style = $srcSetSettings[MediaTwigToolsInterface::OPTION_STYLE];
        }
        else {
          $style = $this->mediaTwigToolsConfig->getImageStyleStorage()
            ->load($srcSetSettings[MediaTwigToolsInterface::OPTION_STYLE]);
        }
      }
      else {
        $style = NULL;
      }
    }

    return $this->replace(
      '@dir/@mediaDir/@style/@rules/@fid_@file.@extension',
      [
        '@dir' => $dirPath['dirname'],
        '@mediaDir' => $this->getMediaDirname(),
        '@style' => $style ? $style->id() : $this->getNoStyleName(),
        '@rules' => $this->getRulesDirnameFromRulesData($rules),
        '@file' => $urlPath['filename'],
        '@extension' => $urlPath['extension'],
        '@fid' => $fileId,
      ]
    );
  }

  /**
   * Scale and crop image.
   *
   * @param \Drupal\file\FileInterface $file
   *   THe file.
   * @param int $width
   *   THe width.
   * @param int $height
   *   The height.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getScaledAndCroppedUrl(FileInterface $file, $width, $height, array $options = []) {
    $rules = [
      [
        "id" => "image_scale_and_crop",
        "data" => [
          "width" => $width,
          "height" => $height,
        ],
      ],
    ];

    return $this->getRuledUrlFromMedia($file, $rules, $options);
  }

  /**
   * Return ruled url.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file.
   * @param array $rules
   *   THe rules.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getRuledUrlFromMedia(FileInterface $file, array $rules, array $options = []) {
    $srcSettings = [
      MediaTwigToolsInterface::OPTION_RULES => $rules,
    ];

    return $this->getImageUrlFromSettings($file->id(), $srcSettings, [], $options);
  }

  /**
   * Build original image.s.
   *
   * @param \Drupal\file\FileInterface|null $file
   *   The file.
   */
  public function buildOriginalImage(FileInterface $file = NULL): void {
    if ($file) {
      $url = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
      $this->httpClient->get($url);
    }
  }

  /**
   * Return the styled url from file Id.
   *
   * @param string|int $file_id
   *   THe file id.
   * @param mixed $style
   *   The style.
   * @param bool $relative
   *   THe url must be relative or not.
   *
   * @return string|null
   *   The file url.
   */
  public function getStyledUrlFromFileId($file_id, $style = NULL, $relative = TRUE): ?string {
    if ($style) {
      $style = $style instanceof ImageStyleInterface ? $style : $this->mediaTwigToolsConfig->getImageStyleStorage()
        ->load($style);
    }
    $url = NULL;
    /** @var \Drupal\file\Entity\File $file */
    $file = $this->mediaTwigToolsConfig->getFileStorage()->load($file_id);
    if ($file) {
      if ($style instanceof ImageStyleInterface) {
        $url = $this->fileUrlGenerator->generateAbsoluteString($style->buildUri($file->getFileUri()));
      }
      else {
        $url = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
      }
    }

    if ($relative) {
      $url = $this->fileUrlGenerator->transformRelative($url);
    }

    return $url;
  }

}
