<?php

namespace Drupal\media_twig_tools\Services;

use Drupal\file\FileInterface;

/**
 * This class builds image from settings.
 *
 * @package Drupal\media_twig_tools\Services
 */
interface ImageBuilderInterface {

  /**
   * Service name.
   *
   * @const string
   */
  const SERVICE_NAME = 'media_twig_tools.image_builder';

  /**
   * Rules separator.
   *
   * @const string
   */
  const RULES_SEPARATOR = '@';

  /**
   * Return the url image from settings.
   *
   * @param string $fileId
   *   The file id.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromSettings($fileId, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce');

  /**
   * Quick replace.
   *
   * @param string $source
   *   The source string.
   * @param array $replace
   *   The replacement [$needle => $replacement].
   *
   * @return array|string|string[]
   *   The string replaced.
   */
  public function replace($source, array $replace);

  /**
   * Return the rules from the dirname.
   *
   * @param string $path
   *   THe path.
   *
   * @return array
   *   The rules.
   */
  public function getRulesFromPathString($path);

  /**
   * Return rules data.
   *
   * @param array $srcSettings
   *   The src settings.
   *
   * @return array
   *   The rules data.
   */
  public function getRulesData(array $srcSettings);

  /**
   * Return the dirname according to rules data.
   *
   * @param array $rulesData
   *   The rules data.
   *
   * @return string
   *   The rules dirname.
   */
  public function getRulesDirnameFromRulesData(array $rulesData);

  /**
   * Return true if $path is an image builder route.
   *
   * @param string $path
   *   THe path.
   *
   * @return bool
   *   The status of the image.
   */
  public function isImageBuilderRoute(string $path);

  /**
   * Return the build image dirname.
   *
   * @return string
   *   THe media dirname.
   */
  public function getMediaDirname();

  /**
   * Return the no style name.
   *
   * @return string
   *   THe no style name.
   */
  public function getNoStyleName();

  /**
   * Generat the image and return the generator.
   *
   * @param string $path
   *   The image path.
   *
   * @return \Drupal\media_twig_tools\Generators\ImageGenerator
   *   The Image Generator.
   *
   * @throws \Exception
   */
  public function generateImage(string $path);

  /**
   * Return image builder data from image path.
   *
   * @param string $path
   *   The path.
   *
   * @return array
   *   The data.
   *
   * @throws \Exception
   */
  public function getDataFromPath($path);

  /**
   * Return the url image from file and settings.
   *
   * @param string $fileId
   *   The file id.
   * @param string $url
   *   The file url.
   * @param array $srcSettings
   *   The source settings.
   * @param array $srcSetSettings
   *   The source set settings.
   * @param array $options
   *   The options.
   * @param string $rule
   *   The rule.
   *
   * @return array|string|string[]|null
   *   The image path.
   */
  public function getImageUrlFromFileAndSettings($fileId, $url, array $srcSettings, array $srcSetSettings, array $options, $rule = 'reduce');

  /**
   * Scale and crop image.
   *
   * @param \Drupal\file\FileInterface $file
   *   THe file.
   * @param int $width
   *   THe width.
   * @param int $height
   *   The height.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getScaledAndCroppedUrl(FileInterface $file, $width, $height, array $options = []);

  /**
   * Return ruled url.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file.
   * @param array $rules
   *   THe rules.
   * @param array $options
   *   The options.
   *
   * @return array|string|string[]|null
   *   The url.
   */
  public function getRuledUrlFromMedia(FileInterface $file, array $rules, array $options = []);

  /**
   * Build original image.s.
   *
   * @param \Drupal\file\FileInterface|null $file
   *   The file.
   */
  public function buildOriginalImage(FileInterface $file = NULL): void;

  /**
   * Return the styled url from file Id.
   *
   * @param string|int $file_id
   *   THe file id.
   * @param mixed $style
   *   The style.
   * @param bool $relative
   *   THe url must be relative or not.
   *
   * @return string|null
   *   The file url.
   */
  public function getStyledUrlFromFileId($file_id, $style = NULL, $relative = TRUE): ?string;

}
