<?php

namespace Drupal\media_twig_tools\Services;

use Drupal\media\MediaInterface;

/**
 * Provides twig tools for facilitate media integration.
 *
 * - Render the image tag directly :
 *    {{ imgFromMedia(content.field_media_1, {style:'my_style', class:'class1
 * class2', title:'My title', {attributes: {'data-attr-1':'1']}}) }}
 *
 * @package Drupal\media_twig_tools\Services
 */
interface MediaTwigToolsInterface {

  /**
   * OPTION style.
   *
   * @const string
   */
  const OPTION_STYLE = 'style';

  /**
   * OPTION nom du champ.
   *
   * @const string
   */
  const OPTION_FIELD_NAME = 'field_name';

  /**
   * OPTION class.
   *
   * @const string
   */
  const OPTION_CLASS = 'class';

  /**
   * OPtion title.
   *
   * @const string
   */
  const OPTION_TITLE = 'title';

  /**
   * OPTION dimension.
   *
   * @const string
   */
  const OPTION_DIMENSION = 'dimension';

  /**
   * OPTION Settings.
   *
   * @const string
   */
  const OPTION_SETTINGS = 'settings';

  /**
   * OPTION relative url.
   *
   * @const strings
   */
  const OPTION_RELATIVE = 'relative';

  /**
   * OPTION attribute.
   *
   * @const string
   */
  const OPTION_ATTRIBUTES = 'attributes';

  /**
   * OPTION rules.
   *
   * @const string
   */
  const OPTION_RULES = 'rules';

  /**
   * OPTION no-size.
   *
   * @const string
   */
  const OPTION_NO_SIZE = 'no_size';

  /**
   * OPTION Override src.
   *
   * @const string
   */
  public const OPTION_OVERRIDE_SRC = 'data-override-src';

  /**
   * Service name.
   *
   * @const string
   */
  const SERVICE_NAME = 'media_twig_tools.media_twig_tools';

  /**
   * Return the default options.
   *
   * @param array $options
   *   The user options.
   *
   * @return array
   *   THe merged options
   */
  public function getDefaultOptions(array $options);

  /**
   * Return an image build array from a media.
   *
   * @param mixed $media_params
   *   The media or the media build array.
   * @param array $options
   *   The options.
   *
   * @return array
   *   The image build array.
   */
  public function imgFromMedia($media_params, array $options = []): array;

  /**
   * Return the list of media entities from a twig parameter.
   *
   * @param mixed $media_params
   *   The media params.
   *
   * @return array
   *   The media entity list.
   */
  public function getMediaEntityFromTwigParam($media_params): array;

  /**
   * Return the media url.
   *
   * @param mixed $media_params
   *   THe media.
   * @param array $options
   *   The otpions.
   *
   * @return array
   *   The media Urls.
   */
  public function getMediaUrl($media_params, array $options = []): array;

  /**
   * Return the styled url from file Id.
   *
   * @param string|int $file_id
   *   THe file id.
   * @param mixed $style
   *   The style.
   * @param bool $relative
   *   THe url must be relative or not.
   *
   * @return string|null
   *   The file url.
   */
  public function getStyledUrlFromFileId($file_id, $style = NULL, $relative = TRUE): ?string;

  /**
   * Return the file id from media.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   * @param array $options
   *   The options.
   *
   * @return string
   *   The file id.
   */
  public function getFileIdFromMedia(MediaInterface $media, array $options): string;

  /**
   * Build picture tag from media.
   *
   * @param mixed $media_params
   *   The media params from twig.
   * @param array $options
   *   The options.
   *
   * @return array
   *   The build array.
   */
  public function pictureFromMedia($media_params, array $options): array;

  /**
   * Return the list of built files from the media.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media.
   *
   * @return array
   *   The files path.
   */
  public function getBuiltFilesFromMedia(MediaInterface $media): array;

  /**
   * Clear built files.
   *
   * @param \Drupal\media\MediaInterface $media
   *   Media.
   */
  public function clearBuiltFilesFromMedia(MediaInterface $media): void;

}
